
#include "GWt_bend.h"
#include "mixedTools.h"

#include <Wt/WText>
#include <Wt/WLineEdit>
#include <Wt/WBreak>
#include <Wt/WDialog>
#include <Wt/WPushButton>

GWt_bend::GWt_bend(abstractElement* elem ) : GWt_abstractElement(elem)
{
    
    string* param = beginDialog();
    
    new WText("angle (deg.) : ",dialog_->contents());
    angleEdit_ = new WLineEdit( param->c_str(), dialog_->contents());
    new WBreak(dialog_->contents());
    
    new WText("radius (m) : ",dialog_->contents());
    radiusEdit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    new WBreak(dialog_->contents());
    
    new WText("momentum (MeV/c) : ",dialog_->contents());
    momentumEdit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    new WBreak(dialog_->contents());
    
    new WText("length (cm) : ",dialog_->contents());
    xlengthEdit_ = new WLineEdit("0", dialog_->contents());
    xlengthEdit_->setDisabled(true);
    new WBreak(dialog_->contents());
    
    new WText("aperture (cm) : ",dialog_->contents());
    apertureEdit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    apertureEdit_->setDisabled(true);
    new WBreak(dialog_->contents());
    
    new WText("pole face rotations (deg.) entrance : ",dialog_->contents());
    beta1Edit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    new WText("exit : ",dialog_->contents());
    beta2Edit_ = new WLineEdit( (++param)->c_str(), dialog_->contents());
    new WBreak(dialog_->contents());
    
    WPushButton *submit = new WPushButton("OK",dialog_->contents());
    submit->clicked().connect(dialog_, &Wt::WDialog::accept);
    dialog_->finished().connect(this, &GWt_bend::dialogDone);
    
    // make image
    image_ = new WImage("htdocs/bend.jpg",this);
    
    // activate slots
    image_->clicked().connect(this,&GWt_abstractElement::clicked);
    
    
}

void GWt_bend::dialogDone()
{
    unsigned nbParam = abstractElement_->getNbParams();
    string* envoi = new string[nbParam+1];
    int compteur = -1;
    envoi[++compteur] = string(mixedTools::intToString(nbParam) );
    envoi[++compteur] = labelEdit_->text().toUTF8();
    cout << " GWt_bend::dialogDone() j'envoie " << envoi[compteur] << endl;
    envoi[++compteur] = angleEdit_->text().toUTF8();
    envoi[++compteur] = radiusEdit_->text().toUTF8();
    envoi[++compteur] = momentumEdit_->text().toUTF8();
    envoi[++compteur] = apertureEdit_->text().toUTF8();
    envoi[++compteur] = beta1Edit_->text().toUTF8();
    envoi[++compteur] = beta2Edit_->text().toUTF8();
    
    if ( compteur !=  (int)nbParam ) {
        cerr << " GWt_bend::dialogDone() pb nb de parametres, compteur = " << compteur << " nbParam= " << nbParam << endl;
        return;
    }
    abstractElement_->setParametersString(envoi);
    updateLabelWidget();
    delete [] envoi;
}


WImage* GWt_bend::getImage() {
    return image_;
}


WString GWt_bend::print(){
    return WString(abstractElement_->print());
}
