#include <iostream>

#include "GWt_elementLigneFaisceau.h"
#include "nomdElements.h"
#include "GWt_console.h"
#include "abstractElement.h"
#include "GWt_rfgun.h"
#include "GWt_drift.h"
#include "GWt_cell.h"
#include "GWt_bend.h"
#include "GWt_soleno.h"
#include "GWt_beam.h"
#include "GWt_fit.h"


#include <Wt/WCssDecorationStyle>
#include <Wt/WSignalMapper>
#include <Wt/WText>
#include <Wt/WImage>
#include <Wt/WHBoxLayout>
#include <Wt/WGridLayout>
#include <Wt/WLength>
#include <Wt/WVBoxLayout>
#include <Wt/WApplication>

#define PSPA_DEBUG 1

GWt_elementLigneFaisceau::GWt_elementLigneFaisceau()
: WContainerWidget(),
firstTimeInsertMessage_(NULL),
dropZoneLeft_(NULL),
dropZoneRight_(NULL),
abstractElement_(NULL)
{
    // set the css name
    setObjectName("elementLigneFaisceau");
    
    initialize_ligneFaisceau(NULL);
}


GWt_elementLigneFaisceau::GWt_elementLigneFaisceau(abstractElement* abstractElement)
: WContainerWidget(),
firstTimeInsertMessage_(NULL),
dropZoneLeft_(NULL),
dropZoneRight_(NULL),
abstractElement_(NULL),
elementName_(NULL)
{
    initialize_ligneFaisceau(abstractElement);
}


void GWt_elementLigneFaisceau::initialize_ligneFaisceau(abstractElement* abstractElement)
{
    
    cout<<"new GWt_elementLigneFaisceau : "<< this << endl;
    
    // set layout
    //     +----------+--------+---------+
    //     |   (--->) |  ELEM  | (-----) |
    //     +----------+--------+---------+
    //     |             NAME            |
    //     +----------+--------+---------+
    
    
    WGridLayout *gridLayout = new WGridLayout();
    
    gridLayout->setContentsMargins(0,0,0,0);
    gridLayout->setHorizontalSpacing	(0);
    gridLayout->setVerticalSpacing	(0);
    
    // add first line :  initialize arrow and link images
    dropZoneLeft_ = new GWt_dropZoneLigneFaisceau();
    dragElementZone_ = new WContainerWidget();
    dropZoneRight_ = new GWt_dropZoneLigneFaisceau();
    
    // change object name in order to manage css :
    dragElementZone_->setStyleClass("beamLineIcon");
    dropZoneRight_->resize(10,30);
    
    
    WContainerWidget* arrow = new WContainerWidget();
    new WImage(arrow);
    arrow->resize(10,30);
    
    arrow->setStyleClass("elementLigneFaiseau_arrow");
    dropZoneLeft_->addWidget(arrow);
    
    gridLayout->addWidget	(dropZoneLeft_,0,0);
    gridLayout->addWidget	(dragElementZone_,0,1);
    gridLayout->addWidget	(dropZoneRight_,0,2);
    
    // add the image if present
    if (abstractElement != NULL) {
        typedElement eType = abstractElement->getNomdElement().getElementType();
        
        switch (  eType ) {
            case RFgun :
                abstractElement_ = new GWt_rfgun(abstractElement);
                break;
            case drift :
                abstractElement_ = new GWt_drift(abstractElement);
                break;
            case cell  :
                abstractElement_ = new GWt_cell(abstractElement);
                break;
            case bend  :
                abstractElement_ = new GWt_bend(abstractElement);
                break;
            case soleno  :
                abstractElement_ = new GWt_soleno(abstractElement);
                break;
            case beam  :
                abstractElement_ = new GWt_beam(abstractElement);
                break;
            case fit  :
                abstractElement_ = new GWt_fit(abstractElement);
                break;
            case inconnu :
                abstractElement_ = NULL;
        }
        if (abstractElement_ != NULL) {
            dragElementZone_->addWidget(abstractElement_->getImage());
            dragElementZone_->setMaximumSize(32,32);
            dragElementZone_->setMinimumSize(32,32);
            dragElementZone_->setToolTip(abstractElement_->print());
        }
    }
    // add second line
    if (abstractElement == NULL) {
        elementName_ = new WText();
    } else {
        elementName_ = new WText(abstractElement->getLabel());
    }
    
    gridLayout->addWidget	(elementName_,1,0,1,3,Wt::AlignCenter |Wt::AlignMiddle);
    
    gridLayout->setColumnStretch (0,1);
    gridLayout->setColumnStretch (2,1);
    
    //  dragElementZone_->addWidget(firstTimeInsertMessage_);
    //  gridLayout->addWidget	(new WText("Drag elements here"),0,0);
    
    // set layout
    setLayout(gridLayout);
    //addWidget(new WText("Drag elements here"));
}


GWt_elementLigneFaisceau::~GWt_elementLigneFaisceau()
{
}


void GWt_elementLigneFaisceau::onClick(GWt_abstractElement* gw)
{
    cout << "mouse key was double clicked on this widget " << gw->getWidget()->id()<< endl;
    
    messageDropZone_ = new GWt_dialog("Question","Are you sure you want to delete the selected item?",GWt_dialog::Warning,true,true);
    
    WSignalMapper<GWt_abstractElement *> *MyMap = new WSignalMapper<GWt_abstractElement *>(this);
    MyMap->mapped().connect(this, &GWt_elementLigneFaisceau::removeDone);
    MyMap->mapConnect(messageDropZone_->finished(),gw);
    messageDropZone_->exec();
}


void GWt_elementLigneFaisceau::removeDone(GWt_abstractElement* gw)
{
    if (messageDropZone_->result() == WDialog::Accepted) {
        // FIXME
        //    removeDropZone();
    }
}




void GWt_elementLigneFaisceau::addElement(GWt_abstractElement* elem){
    
    clear();
    abstractElement_ = elem;
    addWidget(abstractElement_);
    if (static_cast <WContainerWidget*> (parent())) {
        WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
        WBoxLayout* layout = ((WBoxLayout*)(wc->layout()));
        layout->setStretchFactor(this,0);
    }
    
    
    firstTimeInsertMessage_ = NULL;
    dropZoneLeft_ = NULL;
}

std::string GWt_elementLigneFaisceau::getElementNumberInBeamLine(nomdElements elem){
    string str = "";
    
    if (static_cast <WContainerWidget*> (parent())) {
        WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
        for (int a=0; a<wc->count(); a++) {
            if (static_cast <GWt_abstractElement*> (wc->widget (a))) {
                //        GWt_abstractElement* ab = static_cast <GWt_abstractElement*> (wc->widget (a));
                /*
                 printf("1--->%d\n",ab->getElement());
                 printf("2--->%d\n",ab->getElement()->getLenghtOfElement());
                 printf("3--->%d\n",ab->getElement()->getName().getElementType());
                 printf("4--->%d\n",elem.getElementType());
                 
                 if (static_cast <GWt_abstractElement*> (wc->widget (a))->getElement()->getName().getElementType() == elem.getElementType()) {
                 pspa_->addConsoleMessage("type trouv");
                 } else {
                 pspa_->addConsoleMessage("type trouv NOT ");
                 }
                 */
                
            }
        }
        
        int n = 999999;
        
        std::stringstream stream;
        stream << n ;
        std::string number(stream.str());
        
        if(n < 10) {
            str= elem.getElementLabel()+"0"+number;
        } else {
            str= elem.getElementLabel()+number;
        }
    }
    
    return str;
}

void GWt_elementLigneFaisceau::setElementName(std::string label) {
    elementName_->setText(label);
}

