#include <iostream>

#include "GWt_elementLigneFaisceau.h"
#include "GWt_ligneFaisceau.h"
#include "GWt_drift.h"
#include "GWt_rfgun.h"
#include "GWt_cell.h"
#include "GWt_soleno.h"
#include "GWt_bend.h"
#include "GWt_dialog.h"

#include "mixedTools.h"

#include <Wt/WVBoxLayout>


GWt_LigneFaisceau::GWt_LigneFaisceau(PspaApplication* ps) :
WContainerWidget(),
pspa_(ps),
nObjets_(NULL)
{
    
    cout<<"GWt_LigneFaisceau::GWt_LigneFaisceau() %d"<< this << "<---------------"<< endl;
    
    ligneFaisceauLayout_ = new WHBoxLayout();
    //  decorationStyle().setBackgroundColor (WColor(245,245,245));
    setLayout(ligneFaisceauLayout_);
    ligneFaisceauLayout_->setContentsMargins(0,0,0,0);
    ligneFaisceauLayout_->setSpacing	(0);
    
    int nElts= nomdElements::getNumberOfElements();
    nObjets_= new Compteur[nElts];
    
    // set the object name
    setObjectName("ligneFaisceau");
    
    
    // add the first drop zone
    decorationStyle().setBorder (WBorder (WBorder::Dotted));
    GWt_dropZoneLigneFaisceau* allDropZone= new GWt_dropZoneLigneFaisceau();
    WVBoxLayout* fit = new WVBoxLayout();
    allDropZone->setLayout(fit);
    
    fit->addWidget(new WText("Drag elements here...."),1,Wt::AlignCenter |Wt::AlignMiddle);
    ligneFaisceauLayout_->addWidget(allDropZone,1);
}



void GWt_LigneFaisceau::restoreElementCollectionFromDataManager()
{
    
    cout<<"GWt_LigneFaisceau::restoreElementCollection()"<<endl;
    
    ligneFaisceauLayout_->clear();
    
    delete[] nObjets_;
    int nElements= nomdElements ::getNumberOfElements();
    nObjets_= new Compteur[nElements];
    
    int nbElem = pspa_->getDataManager()->beamLineSize();
    unsigned int k;
    
    for(k = 0; k < (unsigned)nbElem; k++)
    {
        abstractElement* ptr = pspa_->getDataManager()->getCollection()->getElementPointerFromIndex(k);
        
        nObjets_[ptr->getNomdElement().getElementType()].incr();

        ligneFaisceauLayout_->addWidget(new GWt_elementLigneFaisceau(ptr),1);
        
    }
    manageLineFaisceauLayout();
    
    pspa_->updateSelections();
}


void GWt_LigneFaisceau::addElement(typedElement eType, GWt_dropZoneLigneFaisceau* dropContainer ){
    
    cout << " Entree addElement element " << nomdElements::getLabelFromType(eType) << endl;
    
    int nElements= nomdElements ::getNumberOfElements();
    nObjets_= new Compteur[nElements];
    
    // 2 cases :
    // -dropContainer is an element of the ligneFaisceauLayout_
    // -dropContainer is an element of the GWt_elementLigneFaisceau_
    
    if(eType == inconnu) {
        cout << "GWt_LigneFaisceau::element type " << eType << " inconnu " << endl;
        return;
    }

    // add the element to the datamanager
    abstractElement* ptr = pspa_->getDataManager()->addElement(eType);
    
    // first time we insert an element :
    GWt_elementLigneFaisceau* firstElemIsElement = dynamic_cast <GWt_elementLigneFaisceau*> (ligneFaisceauLayout_->itemAt(0)->widget());
    if ((ligneFaisceauLayout_->count() == 1) && (!firstElemIsElement)) {
        int index = indexOf (dropContainer);
        if (index != -1) { // found
            // remove global drop zone
            ligneFaisceauLayout_->clear();
            // add first element
            ligneFaisceauLayout_->addWidget(new GWt_elementLigneFaisceau(ptr),1);
            // remove arrow from begin and end
            
        } else { // error
            return;
        }
    } else {
        int index = -1;
        for (int a=0; a< ligneFaisceauLayout_->count(); a++) {
            GWt_elementLigneFaisceau* elem = static_cast <GWt_elementLigneFaisceau*> (ligneFaisceauLayout_->itemAt(a)->widget ());
            if (elem) {
                if (elem->getDropZoneLeft() == dropContainer) {
                    index = a;
                }
                if (elem->getDropZoneRight() == dropContainer) {
                    index = a+1;
                }
            }
            
        }
        
        
        //  ligneFaisceauLayout_->addWidget(new GWt_elementLigneFaiseau(ptr),0);
        if (index != -1) {
            nObjets_[ptr->getNomdElement().getElementType()].incr();
            ligneFaisceauLayout_->insertWidget(index,new GWt_elementLigneFaisceau(ptr),1);
        }
    }
    manageLineFaisceauLayout();
    
}


void GWt_LigneFaisceau::manageLineFaisceauLayout() {
    // special : hide/setVisible arrow :
    for (int a=0; a< ligneFaisceauLayout_->count(); a++) {
        GWt_elementLigneFaisceau* elem = static_cast <GWt_elementLigneFaisceau*> (ligneFaisceauLayout_->itemAt(a)->widget ());
        if (elem) {
            if ((a == 0) && (a == (ligneFaisceauLayout_->count()-1))) {
                elem->getDropZoneLeft()->setStyleClass("elementLigneFaiseau_transparent");
                elem->getDropZoneRight()->setStyleClass("elementLigneFaiseau_transparent");
                elem->getDropZoneLeft()->widget(0)->setHidden(true);
                printf(" - -  %d\n",a);
            } else if (a == 0){
                elem->getDropZoneLeft()->setStyleClass("elementLigneFaiseau_transparent");
                elem->getDropZoneRight()->setStyleClass("elementLigneFaiseau_line");
                elem->getDropZoneLeft()->widget(0)->setHidden(true);
                printf(" - D %d\n",a);
            } else if (a == (ligneFaisceauLayout_->count()-1)){
                elem->getDropZoneLeft()->setStyleClass("elementLigneFaiseau_line");
                elem->getDropZoneRight()->setStyleClass("elementLigneFaiseau_transparent");
                elem->getDropZoneLeft()->widget(0)->setHidden(false);
                
                printf(" L - %d\n",a);
            } else {
                elem->getDropZoneLeft()->setStyleClass("elementLigneFaiseau_line");
                elem->getDropZoneRight()->setStyleClass("elementLigneFaiseau_line");
                elem->getDropZoneLeft()->widget(0)->setHidden(false);
                printf(" L D %d\n",a);
            }
        }
    }
    
}

